@@ -1,9 +1,10 @@
 // file      : xsde/cxx/parser/non-validating/string-stl.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/parser/non-validating/string-stl.hxx>
+#include <xsde/cxx/parser/non-validating/string-common.hxx>
 
 namespace xsde
 {
@@ -22,12 +23,22 @@ namespace xsde
         void string_pimpl::
         _characters (const ro_string& s)
         {
-          str_ += s;
+          if (_facets ().whitespace_ == 2 && str_.size () == 0)
+          {
+            ro_string tmp (s.data (), s.size ());
+
+            if (trim_left (tmp) != 0)
+              str_ += tmp;
+          }
+          else
+            str_ += s;
         }
 
         std::string string_pimpl::
         post_string ()
         {
+          string_common::process_facets (str_, _facets ());
+
           std::string r;
           r.swap (str_);
           return r;
