@@ -1,11 +1,12 @@
 // file      : xsde/cxx/parser/validating/string.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/config.hxx>
 
 #include <xsde/cxx/parser/validating/string.hxx>
+#include <xsde/cxx/parser/validating/string-common.hxx>
 
 namespace xsde
 {
@@ -29,12 +30,35 @@ namespace xsde
         void string_pimpl::
         _characters (const ro_string& s)
         {
+          if (_facets ().whitespace_ == 2 && str_.size () == 0)
+          {
+            ro_string tmp (s.data (), s.size ());
+
+            if (trim_left (tmp) != 0)
+            {
 #ifdef XSDE_EXCEPTIONS
-          str_.append (s.data (), s.size ());
+              str_.append (tmp.data (), tmp.size ());
 #else
-          if (str_.append (s.data (), s.size ()))
-            _sys_error (sys_error::no_memory);
+              if (str_.append (tmp.data (), tmp.size ()))
+                _sys_error (sys_error::no_memory);
 #endif
+            }
+          }
+          else
+          {
+#ifdef XSDE_EXCEPTIONS
+            str_.append (s.data (), s.size ());
+#else
+            if (str_.append (s.data (), s.size ()))
+              _sys_error (sys_error::no_memory);
+#endif
+          }
+        }
+
+        void string_pimpl::
+        _post ()
+        {
+          string_common::validate_facets (str_, _facets (), _context ());
         }
 
         char* string_pimpl::
