@@ -1,6 +1,6 @@
 // file      : xsde/cxx/serializer/genx/document.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/config.hxx>
@@ -64,6 +64,7 @@ namespace xsde
 
         // document_simpl
         //
+        const document_simpl::flags document_simpl::pretty_print = 0x01;
 
 	document_simpl::
 	~document_simpl ()
@@ -85,14 +86,7 @@ namespace xsde
 #ifdef XSDE_POLYMORPHIC
           root_static_type_ = st;
 #endif
-          // Leave root_ns_ uninitialized.
-          //
-#ifdef XSDE_EXCEPTIONS
-          root_name_.assign (name);
-#else
-          if (root_name_.assign (name))
-            error_ = error (sys_error::no_memory);
-#endif
+          init_root_name (0, name);
         }
 
         document_simpl::
@@ -112,13 +106,7 @@ namespace xsde
 #ifdef XSDE_POLYMORPHIC
           root_static_type_ = st;
 #endif
-#ifdef XSDE_EXCEPTIONS
-          root_ns_.assign (ns);
-          root_name_.assign (name);
-#else
-          if (root_ns_.assign (ns) || root_name_.assign (name))
-            error_ = error (sys_error::no_memory);
-#endif
+          init_root_name (ns, name);
         }
 
 #ifdef XSDE_STL
@@ -136,14 +124,7 @@ namespace xsde
 #ifdef XSDE_POLYMORPHIC
           root_static_type_ = st;
 #endif
-          // Leave root_ns_ uninitialized.
-          //
-#ifdef XSDE_EXCEPTIONS
-          root_name_.assign (name.c_str (), name.size ());
-#else
-          if (root_name_.assign (name.c_str (), name.size ()))
-            error_ = error (sys_error::no_memory);
-#endif
+          init_root_name (0, name.c_str ());
         }
 
         document_simpl::
@@ -163,18 +144,37 @@ namespace xsde
 #ifdef XSDE_POLYMORPHIC
           root_static_type_ = st;
 #endif
-#ifdef XSDE_EXCEPTIONS
-          root_ns_.assign (ns.c_str (), ns.size ());
-          root_name_.assign (name.c_str (), name.size ());
-#else
-          if (root_ns_.assign (ns.c_str (), ns.size ()) ||
-              root_name_.assign (name.c_str (), name.size ()))
-            error_ = error (sys_error::no_memory);
-#endif
+          init_root_name (ns.c_str (), name.c_str ());
         }
 #endif // XSDE_STL
 
 
+        void document_simpl::
+        init_root_name (const char* ns, const char* name)
+        {
+          // Leave root_ns_ uninitialized if ns is not specified.
+          //
+          if (ns != 0 && *ns != '\0')
+          {
+#ifdef XSDE_EXCEPTIONS
+            root_ns_.assign (ns);
+#else
+            if (root_ns_.assign (ns))
+            {
+              error_ = error (sys_error::no_memory);
+              return;
+            }
+#endif
+          }
+
+#ifdef XSDE_EXCEPTIONS
+          root_name_.assign (name);
+#else
+          if (root_name_.assign (name))
+            error_ = error (sys_error::no_memory);
+#endif
+        }
+
         void document_simpl::
         add_prefix (const char* p, const char* ns)
         {
@@ -304,9 +304,9 @@ namespace xsde
 #endif
 
         void document_simpl::
-        serialize (std::ostream& os)
+        serialize (std::ostream& os, flags f)
         {
-          serialize (&ostream_write, &ostream_flush, &os);
+          serialize (&ostream_write, &ostream_flush, &os, f);
         }
 #endif // XSDE_IOSTREAM
 
@@ -351,9 +351,9 @@ namespace xsde
 #endif
 
         void document_simpl::
-        serialize (writer& w)
+        serialize (writer& w, flags f)
         {
-          serialize (&write_thunk, &write_bound_thunk, &flush_thunk, &w);
+          serialize (&write_thunk, &write_bound_thunk, &flush_thunk, &w, f);
         }
 
         // Genx write functions.
@@ -427,8 +427,8 @@ namespace xsde
         }
 
         void document_simpl::
-        serialize (write_func w, write_bound_func wb, flush_func f,
-                   void* user_data)
+        serialize (write_func wf, write_bound_func wbf, flush_func ff,
+                   void* user_data, flags f)
         {
           if (xml_serializer_ == 0)
           {
@@ -447,12 +447,15 @@ namespace xsde
 
           writer_info wi;
           wi.user_data = user_data;
-          wi.write = w;
-          wi.write_bound = wb;
-          wi.flush = f;
+          wi.write = wf;
+          wi.write_bound = wbf;
+          wi.flush = ff;
 
           genxSetUserData (xml_serializer_, &wi);
 
+          if (f & pretty_print)
+            genxSetPrettyPrint (xml_serializer_, 2);
+
           genxSender sender;
           sender.send = &genx_write;
           sender.sendBounded = &genx_write_bound;
@@ -486,12 +489,20 @@ namespace xsde
         void document_simpl::
         serialize (genxWriter s)
         {
+          context ctx (s);
+
 #ifndef XSDE_POLYMORPHIC
-          genxStatus e = genxStartElementLiteral (
-            s,
-            reinterpret_cast<constUtf8> (root_ns_.data ()),
-            reinterpret_cast<constUtf8> (root_name_.data ()));
+
+#ifdef XSDE_EXCEPTIONS
+          ctx.start_element (root_ns_.data (), root_name_.data ());
 #else
+          if (!ctx.start_element (root_ns_.data (), root_name_.data ()))
+          {
+            error_ = error (ctx.xml_error ());
+            return;
+          }
+#endif
+#else // XSDE_POLYMORPHIC
           const char* ns = root_ns_.data ();
           const char* n = root_name_.data ();
           const char* dt = 0;
@@ -503,63 +514,62 @@ namespace xsde
             // Call to check sets ns and n if successful.
             //
             if (strcmp (dt, root_static_type_) == 0 ||
-                substitution_map_instance ().check (ns, n, dt))
+                substitution_map_instance ().check (ns, n, dt, 0))
               dt = 0;
           }
 
-          genxStatus e = genxStartElementLiteral (
-            s,
-            reinterpret_cast<constUtf8> (ns),
-            reinterpret_cast<constUtf8> (n));
-
-#endif // XSDE_POLYMORPHIC
-
-          if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+          ctx.start_element (ns, n);
+#else
+          if (!ctx.start_element (ns, n))
           {
-            filter_xml_error (e);
+            error_ = error (ctx.xml_error ());
             return;
           }
+#endif
+#endif // XSDE_POLYMORPHIC
 
           // Add namespace prefixes.
           //
           for (size_t i = 0; i < prefixes_.size (); ++i)
           {
 #ifdef XSDE_STL
-            e = genxAddNamespaceLiteral (
-              s,
-              reinterpret_cast<constUtf8> (prefix_namespaces_[i].c_str ()),
-              reinterpret_cast<constUtf8> (prefixes_[i].c_str ()));
+            const char* p = prefixes_[i].c_str ();
+            const char* ns = prefix_namespaces_[i].c_str ();
 #else
-            e = genxAddNamespaceLiteral (
-              s,
-              reinterpret_cast<constUtf8> (prefix_namespaces_[i]),
-              reinterpret_cast<constUtf8> (prefixes_[i]));
+            const char* p = prefixes_[i];
+            const char* ns = prefix_namespaces_[i];
 #endif
-            if (e != GENX_SUCCESS)
+
+#ifdef XSDE_EXCEPTIONS
+            ctx.declare_namespace (ns, p);
+#else
+            if (!ctx.declare_namespace (ns, p))
               break;
+#endif
           }
 
-          if (e != GENX_SUCCESS)
+#ifndef XSDE_EXCEPTIONS
+          if (ctx.error_type ())
           {
-            filter_xml_error (e);
+            error_ = error (ctx.xml_error ());
             return;
           }
+#endif
 
           // Add the schema location attributes.
           //
           if (schemas_.size () != 0)
           {
-            e = genxAddNamespaceLiteral (
-              s,
-              reinterpret_cast<constUtf8> (xsi),
-              reinterpret_cast<constUtf8> ("xsi"));
-
-            if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+            ctx.declare_namespace (xsi, "xsi");
+#else
+            if (!ctx.declare_namespace (xsi, "xsi"))
             {
-              filter_xml_error (e);
+              error_ = error (ctx.xml_error ());
               return;
             }
-
+#endif
             // First add the xsi:schemaLocation attribute.
             //
             bool start = false;
@@ -577,56 +587,56 @@ namespace xsde
               {
                 if (!start)
                 {
-                  e = genxStartAttributeLiteral (
-                    s,
-                    reinterpret_cast<constUtf8> (xsi),
-                    reinterpret_cast<constUtf8> ("schemaLocation"));
-
-                  if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+                  ctx.start_attribute (xsi, "schemaLocation");
+#else
+                  if (!ctx.start_attribute (xsi, "schemaLocation"))
                     break;
-
+#endif
                   start = true;
                 }
                 else
                 {
-                  e = genxAddCharacter (s, ' ');
-
-                  if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+                  ctx.characters (" ", 1);
+#else
+                  if (!ctx.characters (" ", 1))
                     break;
+#endif
                 }
 
-                e = genxAddText (s, reinterpret_cast<constUtf8> (ns));
-
-                if (e != GENX_SUCCESS)
-                  break;
-
-                e = genxAddCharacter (s, ' ');
-
-                if (e != GENX_SUCCESS)
-                  break;
-
-                e = genxAddText (s, reinterpret_cast<constUtf8> (l));
-
-                if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+                ctx.characters (ns);
+                ctx.characters (" ", 1);
+                ctx.characters (l);
+#else
+                if (!ctx.characters (ns) ||
+                    !ctx.characters (" ", 1) ||
+                    !ctx.characters (l))
                   break;
+#endif
               }
             }
 
-            if (e != GENX_SUCCESS)
+#ifndef XSDE_EXCEPTIONS
+            if (ctx.error_type ())
             {
-              filter_xml_error (e);
+              error_ = error (ctx.xml_error ());
               return;
             }
+#endif
 
             if (start)
             {
-              e = genxEndAttribute (s);
-
-              if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+              ctx.end_attribute ();
+#else
+              if (!ctx.end_attribute ())
               {
-                filter_xml_error (e);
+                error_ = error (ctx.xml_error ());
                 return;
               }
+#endif
             }
 
             // Now add the xsi:noNamespaceSchemaLocation attribute.
@@ -646,52 +656,57 @@ namespace xsde
               {
                 if (!start)
                 {
-                  e = genxStartAttributeLiteral (
-                    s,
-                    reinterpret_cast<constUtf8> (xsi),
-                    reinterpret_cast<constUtf8> ("noNamespaceSchemaLocation"));
-
-                  if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+                  ctx.start_attribute (xsi, "noNamespaceSchemaLocation");
+#else
+                  if (!ctx.start_attribute (xsi, "noNamespaceSchemaLocation"))
                     break;
-
+#endif
                   start = true;
                 }
                 else
                 {
-                  e = genxAddCharacter (s, ' ');
-
-                  if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+                  ctx.characters (" ", 1);
+#else
+                  if (!ctx.characters (" ", 1))
                     break;
+#endif
                 }
 
-                e = genxAddText (s, reinterpret_cast<constUtf8> (l));
-
-                if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+                ctx.characters (l);
+#else
+                if (!ctx.characters (l))
                   break;
+#endif
               }
             }
 
-            if (e != GENX_SUCCESS)
+#ifndef XSDE_EXCEPTIONS
+            if (ctx.error_type ())
             {
-              filter_xml_error (e);
+              error_ = error (ctx.xml_error ());
               return;
             }
+#endif
 
             if (start)
             {
-              e = genxEndAttribute (s);
-
-              if (e != GENX_SUCCESS)
+#ifdef XSDE_EXCEPTIONS
+              ctx.end_attribute ();
+#else
+              if (!ctx.end_attribute ())
               {
-                filter_xml_error (e);
+                error_ = error (ctx.xml_error ());
                 return;
               }
+#endif
             }
           }
 
           // Call the root type serializer to serialize the content.
           //
-          context ctx (s);
 
 #if !defined (XSDE_SERIALIZER_VALIDATION) && defined (XSDE_EXCEPTIONS)
 
@@ -772,10 +787,15 @@ namespace xsde
 #endif // !XSDE_SERIALIZER_VALIDATION && XSDE_EXCEPTIONS
 
 
-          e = genxEndElement (s);
-
-          if (e != GENX_SUCCESS)
-            filter_xml_error (e);
+#ifdef XSDE_EXCEPTIONS
+          ctx.end_element ();
+#else
+          if (!ctx.end_element ())
+          {
+            error_ = error (ctx.xml_error ());
+            return;
+          }
+#endif
         }
 
         void document_simpl::
