@@ -1,6 +1,6 @@
 // file      : xsde/cxx/serializer/validating/id.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <string.h> // strlen
@@ -8,6 +8,11 @@
 #include <xsde/cxx/xml/ncname.hxx>
 
 #include <xsde/cxx/serializer/validating/id.hxx>
+#include <xsde/cxx/serializer/validating/string-common.hxx>
+
+#ifdef XSDE_CUSTOM_ALLOCATOR
+#  include <xsde/cxx/allocator.hxx>
+#endif
 
 namespace xsde
 {
@@ -20,8 +25,15 @@ namespace xsde
         id_simpl::
         ~id_simpl ()
         {
-          if (free_)
-            delete[] const_cast<char*> (value_);
+          if (free_ && value_)
+          {
+            char* v = const_cast<char*> (value_);
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            delete[] v;
+#else
+            cxx::free (v);
+#endif
+          }
         }
 
         void id_simpl::
@@ -36,13 +48,24 @@ namespace xsde
           size_t n = strlen (value_);
 
           if (xml::valid_ncname (value_, n))
-            _characters (value_, n);
+          {
+            if (string_common::validate_facets (
+                  value_, n, _facets (), _context ()))
+            {
+              _characters (value_, n);
+            }
+          }
           else
             _schema_error (schema_error::invalid_id_value);
 
           if (free_)
           {
-            delete[] const_cast<char*> (value_);
+            char* v = const_cast<char*> (value_);
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            delete[] v;
+#else
+            cxx::free (v);
+#endif
             value_ = 0;
           }
         }
