@@ -1,6 +1,6 @@
 // file      : xsde/cxx/serializer/validating/long.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <stdio.h> // sprintf/snprintf
@@ -24,6 +24,24 @@ namespace xsde
         void long_simpl::
         _serialize_content ()
         {
+          // Check facets.
+          //
+          const facets& f = _facets ();
+
+          if (f.min_set_ &&
+              (value_ < f.min_ || (!f.min_inc_ && value_ == f.min_)))
+          {
+            _schema_error (schema_error::value_less_than_min);
+            return;
+          }
+
+          if (f.max_set_ &&
+              (value_ > f.max_ || (!f.max_inc_ && value_ == f.max_)))
+          {
+            _schema_error (schema_error::value_greater_than_max);
+            return;
+          }
+
           // We only need strlen("-9223372036854775808") + 1 characters to
           // hold all representations of (possibly 64-bit) long.
           //
